 LIBRARY IEEE;
 USE IEEE.std_logic_1164.ALL;
 USE IEEE.std_logic_unsigned.ALL;


 ENTITY clkgen IS
 GENERIC
 (
 CHANNEL_DURATION: positive := 128 -- must be 128
 );
 PORT
 (
 -- interface I/O signals
 clk: IN std_logic; -- clock input
 reset: IN std_logic; -- synchronous active-high reset
 -- codec chip clock signals
 mclk: OUT std_logic; -- master clock output to codec
 sclk: OUT std_logic; -- serial data clock to codec
 lrck: OUT std_logic; -- left/right codec channel select
 bit_cntr: OUT std_logic_vector(5 DOWNTO 0);
 subcycle_cntr: OUT std_logic_vector(1 DOWNTO 0)
 );
 END clkgen;

 ARCHITECTURE clkgen_arch OF clkgen IS
 CONSTANT yes: STD_LOGIC := '1';
CONSTANT no: STD_LOGIC := '0';
CONSTANT ready: STD_LOGIC := '1';
CONSTANT overrun: STD_LOGIC := '1';
CONSTANT underrun: STD_LOGIC := '1';
CONSTANT left: STD_LOGIC := '0';
CONSTANT right: STD_LOGIC := '1';
 SIGNAL lrck_int: std_logic;
 SIGNAL seq: std_logic_vector(7 DOWNTO 0);
 BEGIN
 gen_clock:
 PROCESS(clk,seq,lrck_int)
 BEGIN
 IF (clk'event AND clk='1') THEN
 IF(reset=YES) THEN -- synchronous reset
 seq <= (OTHERS=>'0');
 lrck_int <= LEFT; -- start with left channel of codec
 ELSIF(seq=CHANNEL_DURATION-1) THEN
 seq <= (OTHERS=>'0'); -- reset sequencer every channel period
 lrck_int <= NOT(lrck_int); -- toggle channel sel every period
 ELSE
 seq <= seq+1;
 lrck_int <= lrck_int;
 END IF;
 END IF;
 END PROCESS;
 lrck <= lrck_int; -- output the channel selector to the codec
 mclk <= clk; -- codec master clock equals input clock
 sclk <= seq(1); -- serial data shift clock = 1/4 master clock
 bit_cntr <= seq(7 DOWNTO 2);
 subcycle_cntr <= seq(1 DOWNTO 0);
 END clkgen_arch;
