library IEEE;
use IEEE.std_logic_1164.all;

ENTITY BarVol	is
	Port (
		clk: IN STD_LOGIC; -- 12 MHz clock
		rst: IN STD_LOGIC; -- active-high reset
		mclk: OUT STD_LOGIC; -- master clock to codec
		lrck: OUT STD_LOGIC; -- left/right clock to codec
		sclk: OUT STD_LOGIC; -- serial data shift clock to codec
		sdout: IN STD_LOGIC; -- serial data from codec ADCs
		sdin: OUT STD_LOGIC; -- serial data to codec DACs
		Bars: OUT std_logic_vector(7 downto 0)
		);
end BarVol;		

architecture structural of BarVol is	

component BarLevel 
 	port(clk,rst: in std_logic;
 	     Volume: in std_logic_vector(19 downto 0);
 	     Bar: out std_logic_vector(7 downto 0)
 	     );
end component;
 
component FindVol
	PORT(	clk : IN STD_LOGIC;		
		reset: IN STD_LOGIC;		
		Rin,Lin: in std_logic_vector(19 downto 0);
		Rready,Lready: in std_logic;
		result_L, result_R:OUT STD_LOGIC_VECTOR(19 DOWNTO 0);
		complete: out std_logic		
	);
END component;

component loopback
PORT
(
clk: IN STD_LOGIC; -- 12 MHz clock
rst: IN STD_LOGIC; -- active-high reset
mclk: OUT STD_LOGIC; -- master clock to codec
lrck: OUT STD_LOGIC; -- left/right clock to codec
sclk: OUT STD_LOGIC; -- serial data shift clock to codec
sdout: IN STD_LOGIC; -- serial data from codec ADCs
sdin: OUT STD_LOGIC; -- serial data to codec DACs
s: OUT STD_LOGIC_VECTOR(1 DOWNTO 0); -- LED segments	
bit_cntr: OUT std_logic_vector(5 downto 0);
r_out,l_out: out std_logic_vector(19 downto 0);
lrdy,rrdy: out std_logic
);
END component;

component reg20 
	PORT(	D : IN  std_logic_vector (19 DOWNTO 0);
			Q : OUT std_logic_vector (19 DOWNTO 0);
			Clr, Clk, control :IN std_logic	    	);
END component;


signal slrck,smclk,ssclk,slrdy,srrdy: std_logic;
signal sWcomp,sRcomp,srReq,swReq: std_logic;
signal sLVOL,sRVOL,sr_out,sl_out: std_logic_vector(19 downto 0);
signal sbit_cntr: std_logic_vector(5 downto 0);
signal dumbs: std_logic_vector(1 downto 0);



begin  	
	lbk: loopback port map(clk,rst,smclk,slrck,ssclk,sdout,sdin,dumbs,sbit_cntr,sr_out,sl_out,slrdy,srrdy);	
	FV: FindVol port map(clk,rst,sr_out,sl_out,srrdy,slrdy,sLVOL,sRVOL,swReq);
	barl: BarLevel port map(clk,rst,SLVOL(19 downto 0),Bars);
		
	sclk <= ssclk;
	mclk <= smclk;
	lrck <= slrck;
	
end structural;	

